function [A,lambda,U,C,predparam]=GCCA(X)
% [A,r,U,C,predparam]=GCCA(X)
% generalized canonical correlation analysis
% 
% INPUT
% X         Cell array of data saisir matrices
% 
% OUTPUT
% A         Cell array of canonical coefficients
% lambda    average R2 between consensus scores and canonical scores
% U         Cell array of canonical scores
% C         Matrix of consensus scores
% predparam parameters needed for prediction

ntable=length(X);

T={};
minrank=min(size(X{1}.d));
for i=1:ntable
    predparam.Mean{i}=mean(X{i}.d);
    X{i}.d=X{i}.d-repmat(mean(X{i}.d),size(X{i}.d,1),1);
    [U,S,V]=svd(X{i}.d);
    if size(S,2)>1
        rank=length(diag(S));
    else
        rank=1;
    end
    
    if rank<minrank
        minrank=rank;
    end
    predparam.Std{i}=std(U(:,1:rank));
    T{i}=U(:,1:rank)./repmat(std(U(:,1:rank)),size(X{i}.d,1),1);
    
    predparam.Pblock{i}=S(1:rank,1:rank)*V(:,1:rank)';
end
    
[C,S,V]=svd(cell2mat(T));
C=C(:,1:minrank);
predparam.Ptotal=S(1:minrank,1:minrank)*V(:,1:minrank)';

U={};
A={};
for i=1:ntable
    A{i}=inv(X{i}.d'*X{i}.d)*X{i}.d'*C;
    U{i}=X{i}.d*A{i};
end

lambda=zeros(1,minrank);
for j=1:minrank
for i=1:ntable
    lambda(j)=lambda(j)+(corr(C(:,j),U{i}(:,j)))^2;
end
end
lambda=lambda/ntable;
    
