function model=crossval_POSO_PLS(model,cvi)
%model=crossval_POSO_PLS(model,cvi)

X=model.X;
Y=model.Y;
nObj=size(X{1}.d,1);
nBlocks=length(X);
nY=size(Y.d,2);
nContributions=length(model.options.blockCombinations);


options=model.options;

wb=model.options.wb;

if ischar(cvi)
    %set cv segments
    switch lower(cvi)
        case 'loo'
            cvi=1:nObj;
        case 'r10'
            nn=floor(nObj/10);
            rest=nObj-10*nn;
            cvi=[repmat(1:10,1,nn) 1:rest];
            cvi=cvi(randperm(nObj));
        case 'r20'
            nn=floor(nObj/20);
            rest=nObj-20*nn;
            cvi=[repmat(1:20,1,nn) 1:rest];
            cvi=cvi(randperm(nObj));
        otherwise
            if length(cvi)~=nObj
                error('cvi segments does not match number of samples')
            end
    end
end


Yval=zeros(nObj,nY);
partialYval=zeros(nObj,nY,nContributions+1);
Bi=zeros(size(cell2mat(model.Scores),2),nY,max(cvi));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if wb==1
    H=waitbar(0,['POSO-PLS cross-validation with ' num2str(max(cvi)) ' segments...']);
end


%cross-validation loop
for i=1:max(cvi)
    if wb==1
        waitbar(i/max(cvi),H)
    end
    idx=find(cvi==i);
    idx_in=setdiff(1:nObj,idx);
    
    %keep out and keep in
    Xcal=cell(1,nBlocks);
    Xval=cell(1,nBlocks);
    for j=1:nBlocks
        Xcal{j}=deleterow(X{j},idx);
        Xval{j}=selectrow(X{j},idx);
    end
    Ycal=deleterow(Y,idx);
    Y0(idx,:)=repmat(mean(Ycal.d),length(idx),1);
    
    %calibrate and predict
    
    modi=POSO_PLS(Xcal,Ycal,model.options);
    [Yval(idx,:),partialYval(idx,:,:)]=predict_POSO_PLS(Xval,modi);
    %Bi(:,:,i)=modi.beta;
    
    for j=1:nContributions
        
        [dummy1, dummy2, PP] = procrustes(model.Scores{j}(idx_in,:),modi.Scores{j},'scaling',false);
        
        if iscell(modi.Loadings{j}) %common component(s)
            for k=1:length(modi.Loadings{j})
                if ~isempty(modi.Loadings{j}{k})
                    DiffLoadings{j}{k}(:,1:size(modi.Scores{j},2),i)=(inv(PP.T)*modi.Loadings{j}{k}')'-model.Loadings{j}{k};
                else
                    DiffLoadings{j}{k}=[];
                end
            end
        else %unique component(s)
            DiffLoadings{j}(:,1:size(modi.Scores{j},2),i)=(inv(PP.T)*modi.Loadings{j}')'-model.Loadings{j};
        end
        
    end
    
end


if wb==1
    close(H)
end




model.cvres.predictedY=Yval;
model.cvres.RMSEP=sqrt(mean((model.cvres.predictedY-model.Y.d).^2));
model.cvres.expVar=(1- sum((model.Y.d-model.cvres.predictedY).^2)./sum((Y0-model.Y.d).^2))*100;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Jackknife of loadings
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for i=1:nContributions
    
    if iscell(model.Loadings{i}) %common components
        
        for j=1:length(DiffLoadings{i})
            
            stdLoadings{i}{j}=sqrt(sum(DiffLoadings{i}{j}.^2*nObj/(nObj-1),3));
            t=abs(model.Loadings{i}{j}./stdLoadings{i}{j});
            p{i}{j}=1-tcdf(t,nObj-1);
        end
    else %unique components
        stdLoadings{i}=sqrt(sum(DiffLoadings{i}.^2*nObj/(nObj-1),3));
        t=abs(model.Loadings{i}./stdLoadings{i});
        p{i}=1-tcdf(t,nObj-1);
        
    end
    
end


model.cvres.jk.p=p;
model.cvres.jk.stdLoadings=stdLoadings;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CV-ANOVA
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

p=zeros(nContributions,nY);
rmsep=[];
expvar=[];
varianceMean=sum((model.Y.d-repmat(mean(model.Y.d),nObj,1)).^2);

tab1 = repmat('| ',size(model.ANOVA.table{1},1),1);

for i=1:nY
    partial_res=squeeze(partialYval(:,i,:))-repmat(Y.d(:,i),1,nContributions+1);
    rmsep(:,i)=sqrt(mean((partial_res).^2))';
    expvar(:,i)=(1-sum((partial_res.^2))./varianceMean(i))*100;
    partial_res=abs(partial_res);
    %partial_res=(exp(partial_res.^0.25)-1)/0.25;
    for j=1:nContributions % en anova for hver ny modellcontribution
        [dummyp,table] = anova2(partial_res(:,j:j+1),1,'off');
        p(j,i)=dummyp(1);
    end
    
    %remove any old cv-anova results
    id=strfind(model.ANOVA.table{i}(1,:),'rmsec');
    model.ANOVA.table{i}(:,(id+10):end)=[];
    
    expvar_str = num2str((round(expvar(2:end,i)*10)/10));
    rmsep_str = num2str(rmsep(:,i));
    tab2 = strvcat('Exp Var(%) ',repmat('-',1,max(size(expvar_str,2),11)),' ',expvar_str);
    tab3 = strvcat(' rmsep ',repmat('-',1,max(size(rmsep_str,2),11)),rmsep_str);
    tab4 = strvcat('    p-value',repmat('-',1,max(size(num2str(p(:,i)),2),11)),' ',num2str(p(:,i)));
    model.ANOVA.table{i}=[model.ANOVA.table{i} tab1 tab2 tab3 tab4];
end
model.ANOVA.pvalues=p;




