function model  = label_POSO_PLS_model(model)

%INPUT:
%model: From POSO_PLS
%BlockNames: optional cellarray of labels for the original blocks
%
%OUTPUT
% model with the following fields added, both cell arrays:
% CompLabel; Label for each component
% ContrLabel: Label for each contribution
%
%For component labels without
% common components are labelled C12, C13
% unique components are labelled U1-1, U1-2, U2-1, U2-2 and so on
%
%to be used with plot_POSO_PLS and other types of model summary
%
%Ingunn Berget, Nofima AS, aug. 2013
%TO DO: Option to use blocknames in labels?

%model size     
nCompsComb = cell2mat(model.options.nCompsForEachContribution); %#components each contribution
nContr = length(model.options.nCompsForEachContribution); %nr of blocks/block combintaions with contributo
nComps = length(nCompsComb); %total number of components

%initialisation
CompLabel = cell(nComps,1);
ContrLabel = cell(nContr,1);

%numbering components from contributions
stop = 0;
comb = model.options.blockCombinations;
for c = 1:nContr
   start = stop + 1;
   %common/unique comp
    if length(comb{c})>1
        t = 'C';
    else t = 'U';
    end
    %which blocks
    t = {[t regexprep(num2str(comb{c}),' ','')]};
    ContrLabel(c) = t;
    
    %which comps
    if isequal(t,'C')
    compnr = model.options.nCompsForEachContribution{c};
    else
        compnr = 1:model.options.nCompsForEachContribution{c};
    end
    nc = length(compnr);
    stop = start+nc-1;
    if nc>1
        tnr = strcat(t,'-',num2str(compnr'));
        tnr = regexprep(tnr,' ','');        %removing white spaces
    else
        tnr = t;
    end   
    CompLabel(start:stop) = tnr;
end
model.CompLabel= CompLabel;
model.ContrLabel = ContrLabel;

