function model=mycrossval(model,cvi,wb)
% cvres=mycrossval(model,cvi,wb)
%
% INPUT
% model   Model struct
% cvi     cross-validation segments
%         vector with index corresponding to segment number
%         'loo' = leave-one-out
%         'r10' = 10 random segments
%         'r20' = 20 random segments
% wb      optional. =1 (default): waitbar is displayed
%                   =0 no waitbar
%
% OUTPUT
% model   Model struct with cross-validation results added
%
% Created: 22/10 2008
% Modified: 23/10 2008
% Status:  Works 23/10 2008
%
% Ingrid Mge

[n,A]=size(model.T);

if nargin==2
    wb=1;
end

if ischar(cvi)
    %set cv segments
    switch lower(cvi)
        case 'loo'
            cvi=1:n;
        case 'r10'
            nn=floor(n/10);
            rest=n-10*nn;
            cvi=[repmat(1:10,1,nn) 1:rest];
            cvi=cvi(randperm(n));
        case 'r20'
            nn=floor(n/20);
            rest=n-20*nn;
            cvi=[repmat(1:20,1,nn) 1:rest];
            cvi=cvi(randperm(n));
        otherwise
            if length(cvi)~=n
                error('cvi segments does not match number of samples')
            end
    end
end


X=model.X;
Y=model.Y;
Yval=zeros(n,size(Y.d,2),A+1); % also keep prediction at 0 comp
Bi=zeros(size(X.d,2),size(Y.d,2),A,max(cvi));

preprocX=model.Xpp.meth;
preprocY=model.Ypp.meth;

if wb==1
    H=waitbar(0,['cross-validation with ' num2str(max(cvi)) ' segments...']);
end

%cross-validation loop
for i=1:max(cvi)
    if wb==1
        waitbar(i/max(cvi),H)
    end
    idx=find(cvi==i);
    
    %keep out
    x=selectrow(X,idx);
    y=selectrow(Y,idx);
    
    %keep in
    xx=selectrow(X,setdiff(1:n,idx));
    yy=selectrow(Y,setdiff(1:n,idx));
    
    %calibrate and predict
    
    modi=mypls(xx,yy,A,preprocX,preprocY);
    Yval(idx,:,1)=repmat(mean(yy.d),length(idx),1); %prediction at 0 comp
    Yval(idx,:,2:end)=mypredict(x,modi);
    Bi(:,:,:,i)=modi.B;
    ExpVarX(i,:)=modi.ExpVarX;
end

model.cvres.Yval = squeeze(Yval);
model.cvres.ResVar = squeeze(sum((Yval-repmat(model.Y.d,[1,1,A+1])).^2)/n);
if size(model.cvres.ResVar,2)>1
    model.cvres.ExpVar = ((1-model.cvres.ResVar./repmat(model.cvres.ResVar(:,1),[ 1 A+1]))*100)';
    model.cvres.ResVar=model.cvres.ResVar';
else
    model.cvres.ExpVar = (1-model.cvres.ResVar./repmat(model.cvres.ResVar(1),[A+1 1]))*100;
    %model.cvres.RMSEP = sqrt(mean((model.cvres.Yval(:,2:A+1)-repmat(model.cvres.Yval(:,1),1,A)).^2));
end

model.cvres.RMSEP=sqrt(model.cvres.ResVar);
%model.cvres.ExpVarX=mean(ExpVarX);



model.cvres.Bi=squeeze(Bi);
%Total varianeces equals the average of individual variances

if size(model.cvres.ResVar,2)==1
    crit = [0:A]'*0.02*model.cvres.ResVar(1) + model.cvres.ResVar;
    [~, Aopt] = min(crit); 
else
%     for ii=1:size(model.cvres.ResVar,2)
%     crit = [0:A]'*0.02*model.cvres.ResVar(1,ii) + model.cvres.ResVar(:,ii);
%     [dummy, Aopt(ii)] = min(crit);
%     end
%     Aopt=max(Aopt);
    crit=[0:A]'*0.02*mean(model.cvres.ResVar(1,:)) + mean(model.cvres.ResVar,2);
    [~,Aopt]=min(crit); 
end

model.Aopt = Aopt-1; %reduce by 1 because crit starts at zero components


%Uncertainty test
SB=squeeze(zeros(size(model.X.d,2),size(model.Y.d,2),A));

if size(model.cvres.ResVar,2)==1 %one Y-variable
    for i=1:A %for each component
        if A>1
            SB(:,i)=sum(((squeeze(model.cvres.Bi(:,i,:))-repmat(model.B(:,1,i),1,max(cvi)))/sqrt((n-1)/n)).^2,2);
        elseif A<=1 & size(X.d,2)==1
                    SB(:,i)=sum(((squeeze(model.cvres.Bi)'-repmat(model.B,1,max(cvi)))/sqrt((n-1)/n)).^2,2);
        else
            SB(:,i)=sum(((squeeze(model.cvres.Bi)-repmat(model.B,1,max(cvi)))/sqrt((n-1)/n)).^2,2);
        end
        t=abs(model.B(:,1,i))./sqrt(SB(:,i));
        p(:,i)=1-tcdf(t,n-1);
    end
else %more than one Y-variable
    for i=1:A %for each component
        for j=1:size(model.cvres.ResVar,2) %number of Y-variables
            if A>1
                SB(:,j,i)=sum(((squeeze(model.cvres.Bi(:,j,i,:))-repmat(model.B(:,j,i),1,max(cvi)))/sqrt((n-1)/n)).^2,2);
            else
                SB(:,j,i)=sum(((squeeze(model.cvres.Bi(:,j,:,:))-repmat(model.B(:,j),1,max(cvi)))/sqrt((n-1)/n)).^2,2);
            end
            t=abs(model.B(:,j,i))./sqrt(SB(:,j,i));
            p(:,j,i)=1-tcdf(t,n-1);
        end
    end
end

if wb==1
    close(H)
end



model.cvres.JK_s=sqrt(SB);
model.cvres.JK_p=p;

