function model=mypls(X,Y,A,preprocX,preprocY)
% model=mypls(X,Y,A,preprocX,preprocY)
% 
% 
% INPUT
% X         Descriptor data matrix, saisir structure
% Y         Response data matrix, saisir structure
% A         Maximum number of components
% preprocX  optional. String containing preprocessing method for X
% preprocY  optional. String containing preprocessing method for Y
%           -'Autoscale'
% 
% OUTPUT
% model
% 
% Created: 21/10 2008
% Modified: 30/10 2008
% Status:  Works 28/10 2008, but does not work without centering of X
%
% Ingrid Mge


[nx,px]=size(X.d);
[ny,py]=size(Y.d);

if nx~=ny
    error('Number of rows in X and Y must be the same')
end

%set labels if they are not given in saisir
if  ~isfield(X,'v')
    X.v=num2str((1:px)');
elseif isempty(X.v)
    X.v=num2str((1:px)');
end
if ~isfield(Y,'v')
    Y.v=num2str((1:py)');
elseif isempty(Y.v)
    Y.v=num2str((1:py)');
end
if ~isfield(X,'i')
    X.i=num2str((1:nx)');
elseif isempty(X.i) 
    X.i=num2str((1:nx)');
end
if  ~isfield(Y,'i')
    Y.i=num2str((1:ny)');
elseif isempty(Y.i)
    Y.i=num2str((1:ny)');
end
    



model.type = 'pls regression';
model.X=X; %save raw data
model.Y=Y;

if nargin==5
    X=mypreprocess(X,preprocX);
    model.Xpp=X.pp;
    X=rmfield(X,'pp');

    Y=mypreprocess(Y,preprocY);
    model.Ypp=Y.pp;
    Y=rmfield(Y,'pp');
elseif nargin==4
    X=mypreprocess(X,preprocX);
    model.Xpp=X.pp;
    X=rmfield(X,'pp');

    if isfield(Y,'pp')
        model.Ypp=Y.pp;
        Y=rmfield(Y,'pp');
    else
        Y=mypreprocess(Y,'Mean center');
        model.Ypp=Y.pp;
        Y=rmfield(Y,'pp');
    end
elseif nargin==3
    preprocX='mean center';
    preprocY='mean center';
    
    X=mypreprocess(X,preprocX);
    model.Xpp=X.pp;
    X=rmfield(X,'pp');

    Y=mypreprocess(Y,preprocY);
    model.Ypp=Y.pp;
    Y=rmfield(Y,'pp');
end
    
%Initialize matrizes
W=zeros(px,A);
P=zeros(px,A);
Q=zeros(py,A);
T=zeros(nx,A);
U=zeros(nx,A);
E=zeros(nx,px,A);
F=zeros(nx,py,A);
B=zeros(px,py,A);
Ycal=zeros(ny,py,A+1); % also keep prediction at 0 comp
ResVarX=zeros(A+1,1); %residual variance in X

Ycal(:,:,1)=repmat(model.Ypp.mean,ny,1); %prediction at 0 comp = mean
e=X.d;
f=Y.d;
ResVarX(1)=mean(mean(e.^2));
for i=1:A
    
    [WW,S,V]=svd(e'*f);
    if S(1)<1e-10 %check if full rank is reached
        warning(['maximum rank is ' num2str(i-1) '. Dropping remaining components']);
        %set values for rest of components
        E(:,:,i:A)=repmat(e,[1 1 A-i+1]);
        F(:,:,i:A)=repmat(f,[1 1 A-i+1]);
        B(:,:,i:A)=repmat(B(:,:,i-1),[1 1 A-i+1]);
        Ycal(:,:,i+1:A+1)=repmat(Ycal(:,:,i),[1 1 A-i+1]);
        break
    end
        
    W(:,i)=WW(:,1);
    T(:,i)=e*W(:,i);
    P(:,i)=e'*T(:,i)/(T(:,i)'*T(:,i));
    Q(:,i)=f'*T(:,i)/(T(:,i)'*T(:,i));
    U(:,i)=f*Q(:,i)/(Q(:,i)'*Q(:,i));
    e=e-T(:,i)*P(:,i)';
    f=f-T(:,i)*Q(:,i)';
    E(:,:,i)=e;
    F(:,:,i)=f;
    B(:,:,i)=W(:,1:i)*inv(P(:,1:i)'*W(:,1:i))*Q(:,1:i)';
    Ycal(:,:,i+1)=(X.d*B(:,:,i)).*repmat(model.Ypp.std,nx,1)+ repmat(model.Ypp.mean,nx,1);
    ResVarX(i+1)=mean(mean(e.^2));
   
end

  
model.T=T;
model.U=U;
model.W=W;
model.P=P;
model.Q=Q;
model.E=E;
model.F=F;
model.B=B;
model.Ycal=squeeze(Ycal);
model.ResVar=squeeze(sum((Ycal-repmat(model.Y.d,[1,1,A+1])).^2)/nx);
model.RMSEC = sqrt(model.ResVar);
if py>1
    model.ExpVar=((1-model.ResVar./repmat(model.ResVar(:,1),[ 1 A+1]))*100)';
    model.ResVar=model.ResVar';
else
    model.ExpVar=(1-model.ResVar./repmat(model.ResVar(1),[A+1 1]))*100;
    
end

model.ResVarX=ResVarX;
model.ExpVarX=(1-model.ResVarX./repmat(model.ResVarX(1),[A+1 1]))*100;
if A>0
if py==1 
    crit = [1:A]'*0.02*mean(model.ResVar(1)) + mean(model.ResVar(2:end),2);
else
    crit = [1:A]'*0.02*mean(model.ResVar(1,:)) + mean(model.ResVar(2:end,:),2);
end
else crit=[]; 
end

[dummy, Aopt] = min(crit);
if isempty(Aopt)
    Aopt=0;
end

model.Aopt = Aopt;

