function plotoptions=plot_POSO_PLS(model,plotoptions)
% plotoptions=plot_POSO_PLS(model)
% plot_POSO_PLS(model,plotoptions)

%Modifications, aug 2013 by Ingunn Berget
%some changes for readability in making labels (use format in num2str)
%dealing with one component models (using additional args for diff)
%options to create different types of plots

%modifications march 2014 by Ingrid Mge
% collect options in plotoptions struct with fields:
% plots:            'all' (default),'expVarPie','Predplot' or 'ScoresLoadings'.
% spec:             =1 if data are spectra 
% PCs:              which two components to plot. Default is [1 2]
% samplegroups:     numeric vector or character array defining sample groups
% variablegroups:   cell array containing numeric array or character array
%                   defining variable groups.

if nargin<2
    plotoptions.plots='all';
    plotoptions.spec=0;
    plotoptions.PCs=[1 2];
    plotoptions.Yvar=[];
    plotoptions.samplegroups=[];
    plotoptions.variablegroups=[];
else
    
    
    model=label_POSO_PLS_model(model);
    
    
    if strcmp('all',plotoptions.plots) == 1 | strcmp('expVarPie',plotoptions.plots)
        plotExpVarPie(model)
    end
    
    if strcmp('all',plotoptions.plots) == 1 | strcmp('PredPlot',plotoptions.plots)
        makePredPlot(model,plotoptions.Yvar)
    end
    
    if strcmp('all',plotoptions.plots) == 1 | strcmp('ScoresLoadings',plotoptions.plots)
        plot_scores_loadings(model,plotoptions.spec,plotoptions.samplegroups,plotoptions.variablegroups)
    end
    
    
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%INTERNAL FUNCTIONS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    function plotExpVarY(model)
        %if less than 20 responses:
        %total variance for each response (bar)
        %and pie chart of explained variance for each block combinations  if less
        %than 20 responses
        %for models with only one comp: bar plot
        %
        
        nBlocks=length(model.X);
        nY=size(model.Y.d,2);
        
        %extracting the explained Y variance for selected Y's
        %pr blockcombination/unique block with contribution
        EV=cell2mat(model.expVarY);
        [nContr nY] = size(EV); %ncomps = number of contribtions
        subplotLayouts = [1 1; 1 2; 2 2;3 2;4 2;4 3;5 3; 4 5];
        nSubplots = prod(subplotLayouts,2);
        
        if nContr > 1 %more than one component in model
            
            %if not too many responses
            %plot explained variance for each response
            if nY <= max(nSubplots)
                figure('Name','Explained Y variance pie chart');
                
                %finding layout of subplots
                leftover = nSubplots - nY;
                leftover(leftover < 0) = nY+10;
                [~, selected] = min(leftover);
                nrow = subplotLayouts(selected,1);
                ncol = subplotLayouts(selected,2);
                
                %for each response
                labels = cell(nContr,1);
                for i = 1:nY
                    for c = 1:nContr
                        labels{c} = char(model.ContrLabel{c},[num2str(EV(c,i),'%2.1f') '%']);
                    end
                    subplot(nrow,ncol,i)
                    pie(EV(:,i)/100,labels)
                    title(model.Y.v(i,:))
                end
                colormap(summer)
            end
            
            %plot overall explained variance for each response
            %     totvar = sum(cell2mat(model.expVarY),1);%make sure row sum
            %     if isfield(model,'cvres')
            %         totvar(2,:) = model.cvres.expVar;
            %     end
            %     figure('Name','EXPLAINED Y VARIANCE TOTAL')
            %     bar(totvar'/100)
            %     set(gca,'xticklabel',cellstr(model.Y.v),'xtick',1:nY)
            %     title('EXPLAINED Y VARIANCE TOTAL ')
            %     if isfield(model,'cvres')
            %         legend({'Cal' 'Val'});
            %     end
        else %only  one component with contribution
            figure
            bar(EV)
            set(gca,'xticklabel',cellstr(model.Y.v),'xtick',1:nY)
            title(['EXPLAINED Y VARIANCE ' model.ContrLabel{1}]);
            
        end
    end %function end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Explained variance pie charts
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    function plotExpVarPie(model)
        
        
        
        %extracting the explained variance for each X block
        for i=1:length(model.options.blockCombinations)
            dummy=cell2mat(model.expVarX(i,:));
            EVX(i,:)=dummy(end,:);
            if size(model.Y.d,2)>1 %more than one Y-variable
                EVY(i,:)=mean(model.expVarY{i});
            else
                EVY=cell2mat(model.expVarY);
            end
        end
        
        
        
        nBlocks = length(model.X);
        [nContr nB] = size(model.expVarX); %nContr = number of contribtions
        
        figure('Name','Explained variance pie charts');
        xx=[EVX(1,:); diff(EVX,1,1)];
        for nb=1:nBlocks
            
            subplot(1,nBlocks+1,nb)
            
            %make labels
            xxstr = num2str(xx(:,nb),'%2.1f');
            for c = 1:nContr
                labels{c} = char(model.ContrLabel{c},[xxstr(c,:) '%']);
            end
            labels(xx(:,nb)<1) = {''};
            h=pie(xx(:,nb)/100,labels);
            tit=['BLOCK ' num2str(nb) ' (' num2str(sum(xx(:,nb),1),'%3.1f') '%)'];
            title(tit,'FontWeight','bold')
        end
        
        %plot Y block
        yystr = num2str(EVY,'%2.1f');
        for c = 1:nContr
            labels{c} = char(model.ContrLabel{c},[yystr(c,:) '%']);
        end
        labels(EVY<1) = {''};
        subplot(1,nb+1,nb+1)
        h=pie(EVY/100,labels);
        tit=['Y BLOCK (' num2str(sum(EVY),'%3.1f') '%)'];
        title(tit,'FontWeight','bold')
        colormap(summer)
        
        
        
    end %function

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Predicted versus measured
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    function makePredPlot(model,Yvar)
        
        %model: From POSO_PLS
        %Yvar: Optional input in main function
        %if given, look at only selected y variables
        %if not given, look at all when number is less than 20
        
        EVY=sum(cell2mat(model.expVarY),1);
        nBlocks=length(model.X);
        nY=size(model.Y.d,2);
        
        %for selecting subplot layout
        subplotLayouts = [1 1; 1 2; 2 2;3 2;4 2;4 3;5 3; 4 5];
        nSubplots = prod(subplotLayouts,2);
        
        %select which Y variables to look at
        if isempty(Yvar)
            Yvar = 1:nY;
        end
        MY = selectcol(model.Y,Yvar);
        FY = model.fittedY(:,Yvar);
        CVY = model.cvres.predictedY(:,Yvar);
        nY = length(Yvar);
        EVYs = EVY(:,Yvar);
        expVarVal = model.cvres.expVar(Yvar);
        %if not too many responses
        %plot explained variance for each response
        if nY <= max(nSubplots)
            figure('Name','Predicted versus measured')
            
            %finding layout of subplots
            leftover = nSubplots - nY;
            leftover(leftover < 0) = nY+10;
            [mm selected] = min(leftover);
            nrow = subplotLayouts(selected,1);
            ncol = subplotLayouts(selected,2);
            
            for i=1:nY
                subplot(nrow,ncol,i)
                plot(MY.d(:,i),FY(:,i),'b.')
                hold on
                title(MY.v(i,:))
                if isfield(model,'cvres')
                    plot(MY.d(:,i),CVY(:,i),'r.')
                    str=char(['Cal = ' num2str(round(EVYs(i)*10)/10) '%'],...
                        ['Val = ' num2str(round(expVarVal(i)*10)/10) '%']);
                else
                    str=['Cal = ' num2str(round(EVYs(i)*10)/10) '%'];
                end
                xlabel('Measured')
                ylabel('Predicted')
                if size(str,1)>1
                    legend(str,'Location','Northwest')
                end
                line([min(MY.d(:,i)) max(MY.d(:,i))],...
                    [min(MY.d(:,i)) max(MY.d(:,i))])
            end
        else %too many responses to get informative plot
            R2 = EVY/100;
            if isfield(model,'cvres')
                R2(2,:) = model.cvres.expVar/100;
            end
            figure('Name','R2 for responses (too see which responses are well pred)')
            bar(1:nY,R2');
            xlabel('responses')
            if isfield(model,'cvres')
                legend({'Cal' 'Val'});
            end
            title('R2')
        end
        
    end %function

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Scores and loadings
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    function plot_scores_loadings(model,spec,sample_groups,variable_groups)
        col=colormap(lines);%strvcat('b','m','c','g','k','y');
        col(3,:)=[]; %remove the red color, which is reserved for y loadings
        
        %make one figure for each contribution
        for i=1:length(model.options.blockCombinations) %each contribution
            
            figure('Name',['Scores & Loadings, ' model.ContrLabel{i}]);
            
            nComps=size(model.Scores{i},2);
            
            if nComps==1 %only one component
                
                %scores
                if spec==0
                    subplot(1,2,1)
                else
                    subplot(1,1+length(model.options.blockCombinations{i}),1)
                end
                
                if isempty(sample_groups) %no sample grouping
                    plot(zeros(size(model.Scores{i})),model.Scores{i},'.')
                    hold on
                    text(zeros(size(model.Scores{i}))+0.03,model.Scores{i},model.X{1}.i)
                else
                    sample_grouping(zeros(size(model.Scores{i})),model.Scores{i},sample_groups,model.X{1}.i);
                    colorbar off
                end
                ylabel(model.ContrLabel{i})
                title(['Scores ' model.ContrLabel{i}])
                
                %loadings
                if spec==0
                    subplot(1,2,2)
                    title(['Correlation Loadings ' model.ContrLabel{i}])
                    
                    idx=model.options.blockCombinations{i};
                    for j=1:length(idx)
                        
                        if isempty(variable_groups)
                            h=plot(zeros(size(model.X{idx(j)}.d,2),1),corr(model.Scores{i}(:,1),model.X{idx(j)}.d),'.');
                            set(h,'MarkerEdgeColor',col(idx(j),:))
                            hold on
                            H=text(zeros(size(model.X{idx(j)}.d,2),1),corr(model.Scores{i}(:,1),model.X{idx(j)}.d),model.X{idx(j)}.v,'Color',col(idx(j),:));
                        else
                            sample_grouping(zeros(size(model.X{idx(j)}.d,2),1),corr(model.Scores{i}(:,1),model.X{idx(j)}.d),variable_groups{idx(j)},model.X{idx(j)}.v)
                            colorbar off
                            hold on
                        end
                        
                    end
                    plot(zeros(size(model.Y.d,2),1),corr(model.Scores{i}(:,1),model.Y.d),strcat('.','r'))
                    text(zeros(size(model.Y.d,2),1),corr(model.Scores{i}(:,1),model.Y.d),strcat(upper(model.Y.v)),'Color','r','EdgeColor','r');
                    
                    H=line(get(gca,'xlim'),[0 0]);
                    set(H,'linestyle',':','Color','k')
                    H=line(get(gca,'xlim'),[-sqrt(0.5) -sqrt(0.5)]);
                    set(H,'linestyle',':','Color','k')
                    H=line(get(gca,'xlim'),[sqrt(0.5) sqrt(0.5)]);
                    set(H,'linestyle',':','Color','k')
                else %spec=1
                    
                    idx=model.options.blockCombinations{i};
                    
                    
                    for k=1:length(idx)
                        
                        subplot(1,1+length(idx),1+k)
                        
                        if length(idx)>1
                            plot(str2num(model.X{idx(k)}.v),model.Loadings{i}{idx(k)});
                            axis tight
                            if ~isempty(variable_groups)
                                sample_grouping(str2num(model.X{idx(k)}.v),model.Loadings{i}{idx(k)},variable_groups{idx(k)},repmat(' ',length(model.X{idx(k)}.v),1));
                            end
                        else
                            plot(str2num(model.X{idx(k)}.v),model.Loadings{i});
                            axis tight
                            if ~isempty(variable_groups)
                                sample_grouping(str2num(model.X{idx(k)}.v),model.Loadings{i},variable_groups{idx(k)},repmat(' ',length(model.X{idx(k)}.v),1));
                            end
                        end
                        colorbar off
                        legend off
                        title([model.ContrLabel{i} '-Loadings, Block ' num2str(idx(k))])
                        
                    end
                    
                end
                
            else % more than one component --> scatter plot
                nplot=1;
                
                nPlots=floor(nComps/2);
                compCombinations=[];
                dummy=1:nComps;
                if ~isequal(length(dummy)/2,floor(length(dummy)/2))
                    dummy=[dummy(1:end-1) 1 dummy(end)];
                end
                compCombinations=reshape(dummy,2,length(dummy)/2)';
                
                for j=1:nPlots
                    
                    %scores
                    if spec==0
                        subplot(nPlots,2,nplot)
                    else
                        subplot(nPlots,1+length(model.options.blockCombinations{i}),nplot)
                    end
                    
                    title(['Scores, ' model.ContrLabel{i}])
                    if isempty(sample_groups)
                        plot(model.Scores{i}(:,compCombinations(j,1)),model.Scores{i}(:,compCombinations(j,2)),'.')
                        hold on
                        text(model.Scores{i}(:,compCombinations(j,1)),model.Scores{i}(:,compCombinations(j,2)),model.Y.i)
                        
                    else
                        sample_grouping(model.Scores{i}(:,compCombinations(j,1)),model.Scores{i}(:,compCombinations(j,2)),sample_groups,model.Y.i);
                        colorbar off
                    end
                    
                    %loadings
                    if spec==0
                        subplot(nPlots,2,nplot+1)
                        idx=model.options.blockCombinations{i};
                        for k=1:length(idx)
                            
                            if isempty(variable_groups)
                                h=plot(corr(model.Scores{i}(:,compCombinations(j,1)),model.X{idx(k)}.d),corr(model.Scores{i}(:,compCombinations(j,2)),model.X{idx(k)}.d),'.');
                                set(h,'MarkerEdgeColor',col(idx(k),:))
                                hold on
                                text(corr(model.Scores{i}(:,compCombinations(j,1)),model.X{idx(k)}.d),corr(model.Scores{i}(:,compCombinations(j,2)),model.X{idx(k)}.d),model.X{idx(k)}.v,'Color',col(idx(k),:));
                            else
                                
                                sample_grouping(corr(model.Scores{i}(:,compCombinations(j,1)),model.X{idx(k)}.d),corr(model.Scores{i}(:,compCombinations(j,2)),model.X{idx(k)}.d),variable_groups{idx(k)},model.X{idx(k)}.v)
                                hold on
                                colorbar off
                            end
                        end
                        
                        plot(corr(model.Scores{i}(:,compCombinations(j,1)),model.Y.d),corr(model.Scores{i}(:,compCombinations(j,2)),model.Y.d),'r.')
                        h=text(corr(model.Scores{i}(:,compCombinations(j,1)),model.Y.d),corr(model.Scores{i}(:,compCombinations(j,2)),model.Y.d),strcat(upper(model.Y.v)),'Color','r','EdgeColor','r');
                        %circle 100%
                        [X,Y] = pol2cart(linspace(0,2*pi,100),ones(1,100));
                        plot(X,Y,'k');
                        %circle 50%
                        [X,Y] = pol2cart(linspace(0,2*pi,100),ones(1,100)*sqrt(0.5));
                        plot(X,Y,'k');
                        H=line([0 0],[-1 1]);
                        set(H,'linestyle',':','Color','k')
                        H=line([-1 1],[0 0]);
                        set(H,'linestyle',':','Color','k')
                        title(['Correlation loadings, ' model.ContrLabel{i}])
                        
                    else %spec=1
                        
                        idx=model.options.blockCombinations{i};
                        
                        
                        for k=1:length(idx)
                            subplot(nPlots,1+length(idx),nplot+k)
                            
                            if length(idx)>1
                                plot(str2num(model.X{idx(k)}.v),model.Loadings{i}{idx(k)});
                                axis tight
                                if ~isempty(variable_groups)
                                    for ii=1:size(model.Loadings{i}{idx(k)},2)
                                        sample_grouping(str2num(model.X{idx(k)}.v),model.Loadings{i}{idx(k)}(:,ii),variable_groups{idx(k)},repmat(' ',length(model.X{idx(k)}.v),1));
                                        hold on
                                    end
                                    
                                    
                                end
                                
                            else %one block
                                
                                if ~isempty(variable_groups)
                                    for ii=1:size(model.Loadings{i},2)
                                        sample_grouping(str2num(model.X{idx(k)}.v),model.Loadings{i}(:,ii),variable_groups{idx(k)},repmat(' ',length(model.X{idx(k)}.v),1));
                                        hold on
                                    end
                                    
                                    
                                end
                                plot(str2num(model.X{idx(k)}.v),model.Loadings{i});
                                axis tight
                            end
                            colorbar off
                            legend off
                            title([model.ContrLabel{i} '-Loadings, Block ' num2str(idx(k))])
                        end
                        
                        
                        
                    end
                end
                nplot=nplot+2;
                
            end
        end
        
    end
end %function

