function []=plsplots(model,A1,A2,Yidx,plottype,spec)
% []=plsplots(model1,A1,A2,Yidx,plottype,spec)
%
%
% Plots:
% scores for A1 and A2, explained variance and predicted versus measured at Aopt
% loadings for A1 and A2
% explained variance for all components
% predicted versus measured for at Aopt components
%
% INPUT
% model       model struct from mypls
% A1,A2       PCs to plot for scores and loadings
% Yidx        Optional. Determins which Y-variable to plot
% plottype    Optional.   =1: plots one figure with subplots (default)
%                         =2: plots each plot in a separate figure
% spec        Optional. =1 for spectral data

if nargin==1
    A1=1;
    if size(model.T,2)==1
        A2=1;
    else
        A2=2;
    end
    [dummy,Yidx]=max(model.ExpVar(model.Aopt+1,:));
    plottype=1;
    spec=0;
elseif nargin==3
    [dummy,Yidx]=max(model.ExpVar(model.Aopt+1,:));
    plottype=1;
    spec=0;
elseif nargin==4
    if isempty(Yidx)
        [dummy,Yidx]=max(model.ExpVar(model.Aopt+1,:));
    end
    plottype=1;
    spec=0;
elseif nargin==5
    spec=0;
end

figure;

%Scores
if plottype==1
subplot(2,2,1)
end
plot(model.T(:,A1),model.T(:,A2),'.')
hold on
T1range=max(model.T(:,A1))-min(model.T(:,A1));
text(model.T(:,A1)+T1range/100,model.T(:,A2),model.X.i)
title('Scores')
xlabel(['Component ' num2str(A1)])
ylabel(['Component ' num2str(A2)])
grid on
h=gca;
% if get(h,'Xlim')==0
%     set(h,'Xtick',0)
% else
% set(h,'Xtick',sort([get(h,'Xlim') 0]))
% end
% if get(h,'Ylim')==0
%     set(h,'Ytick',0)
% else
% set(h,'Ytick',sort([get(h,'Ylim') 0]))
% end


%Correlation Loadings
if plottype==1
    subplot(2,2,2)
else
    figure;
end

if spec~=1
hold off
plot(corr(model.T(:,A1),model.X.d),corr(model.T(:,A2),model.X.d),'.')
hold on
H=text(corr(model.T(:,A1),model.X.d)+2/100,corr(model.T(:,A2),model.X.d),model.X.v);
set(H,'Color','b')
plot(corr(model.T(:,A1),model.Y.d),corr(model.T(:,A2),model.Y.d),'r.');
H=text(corr(model.T(:,A1),model.Y.d)+2/100,corr(model.T(:,A2),model.Y.d),model.Y.v);
set(H,'Color','r')
title('Correlation Loadings')
xlabel(['Component ' num2str(A1)])
ylabel(['Component ' num2str(A2)])
%circle 100%
[X,Y] = pol2cart(linspace(0,2*pi,100),ones(1,100));
plot(X,Y,'k');
%circle 50%
[X,Y] = pol2cart(linspace(0,2*pi,100),ones(1,100)*sqrt(0.5));
plot(X,Y,'k');
H=line([0 0],[-1 1]);
set(H,'linestyle',':','Color','k')
H=line([-1 1],[0 0]);
set(H,'linestyle',':','Color','k')

%mark significant variables
if isfield(model,'cvres')
    if model.Aopt>0
        if size(model.Y.d,2)>1 %more than one Y
            dummyp=min(model.cvres.JK_p(:,:,model.Aopt)')';
            dummyidx=find(dummyp<0.05);
        else % one Y
            dummyidx=find(model.cvres.JK_p(:,model.Aopt)<0.05);
        end
plot(corr(model.T(:,A1),model.X.d(:,dummyidx)),corr(model.T(:,A2),model.X.d(:,dummyidx)),'bo')

    end
end

elseif spec==1
if model.Aopt==0
    Aopt=1;
else
    Aopt=model.Aopt;
end
wl=str2num(model.X.v);
if length(wl)~=size(model.X.d,2);
    wl=1:size(model.X.d,2);
end
plot(wl,model.W(:,1:Aopt))
hold on
labels=strcat('Comp',num2str((1:Aopt)'));
legend(labels);
title('Loading weights')
axis tight

if isfield(model,'cvres')
    if model.Aopt>0
                if size(model.Y.d,2)>1 %more than one Y
            dummyp=min(model.cvres.JK_p(:,:,model.Aopt)')';
            dummyidx=find(dummyp<0.05);
        else % one Y
            dummyidx=find(model.cvres.JK_p(:,model.Aopt)<0.05);
        end
    end
    plot(wl(dummyidx),model.W(dummyidx,1:Aopt),'o')
end

    
end


%Explained variance
if plottype==1
    subplot(2,2,3)
else
    figure;
end

plot(0:size(model.T,2),model.ExpVar,'color',[153 204 255]/255)
hold on
if isfield(model,'cvres')
     plot(0:size(model.T,2),model.cvres.ExpVar,'color',[255 204 204]/255);
end
if numel(size(model.Ycal))==3
    
    MeanExpCal=mean(model.ExpVar');
    
else
    MeanExpCal=model.ExpVar;
end
plot(0:size(model.T,2),MeanExpCal,'b','linewidth',2);

if isfield(model,'cvres')
    if numel(size(model.cvres.Yval))==3
        MeanExpVal=mean(model.cvres.ExpVar');
         
    else
        MeanExpVal=model.cvres.ExpVar;
    end
    plot(0:size(model.T,2),MeanExpVal,'r','linewidth',2);
    plot(model.Aopt,MeanExpVal(model.Aopt+1),'or')
    set(gca,'Ylim',[min(MeanExpVal) 100])
end
plot(model.Aopt,MeanExpCal(model.Aopt+1),'o')
title('Explained variance, Total')
xlabel('Components')
H=line([0 size(model.T,2)],[0 0]);
set(H,'color','k')

%Predicted versus measured
if plottype==1
    subplot(2,2,4)
else
    figure;
end
if numel(size(model.Ycal))==3
    plot(model.Y.d(:,Yidx),squeeze(model.Ycal(:,Yidx,model.Aopt+1)),'.')
else
    plot(model.Y.d,squeeze(model.Ycal(:,model.Aopt+1)),'.')
end

if isfield(model,'cvres')
    hold on
    if numel(size(model.cvres.Yval))==3
        plot(model.Y.d(:,Yidx),model.cvres.Yval(:,Yidx,model.Aopt+1),'r.')
        title(strcat(model.Y.v(Yidx,:),',',num2str(model.Aopt),' components'))
    else
        plot(model.Y.d,model.cvres.Yval(:,model.Aopt+1),'r.')
        title(['Predicted vs Measured at ' num2str(model.Aopt) ' components'])
    end
    legend(['R2=' num2str(round(model.ExpVar(model.Aopt+1,Yidx))/100)],['R2=' num2str(round(model.cvres.ExpVar(model.Aopt+1,Yidx))/100)],'Location','NorthWest')
else
    legend(['R2=' num2str(round(model.ExpVar(model.Aopt+1,Yidx))/100)],'Location','NorthWest')
end
line([min(model.Y.d(:,Yidx)) max(model.Y.d(:,Yidx))], [min(model.Y.d(:,Yidx)) max(model.Y.d(:,Yidx))])
xlabel('Measured')
ylabel('Predicted')

