function [predictedY,partial_predictedY]=predict_POSO_PLS(X,model)





nBlocks=length(X);
if nBlocks ~= length(model.X)
    error('Number of blocks in X must be same as in model')
end

nObj=size(X{1}.d,1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%preprocess
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for i=1:nBlocks
    if isequal(model.options.preprocX{i},'mean center')
        X{i}.d=X{i}.d-repmat(mean(model.X{i}.d),nObj,1);
    elseif isequal(model.options.preprocX{i},'autoscale')
        X{i}.d=(X{i}.d-repmat(mean(model.X{i}.d),nObj,1))./repmat(std(model.X{i}.d),nObj,1);
    end
end
 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Estimate components for all blockCombinations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for k=1:length(model.options.blockCombinations) %for each contribution
    idx=model.options.blockCombinations{k}; % index of contributing blocks 
    
    if length(idx)>1 %common component
    %predict scores from local pls models
    if ~exist('local_t','var') %this is the first common component, need to calculate local scores
    for j=1:length(idx) %number of local pls models
        a=model.options.nCompsLocalPls(j); %number of components
        w=model.forprediction{k}.W{j}(:,1:a);
        p=model.forprediction{k}.P{j}(:,1:a);
        local_t{j}=mat2saisir(X{j}.d*w*inv(p'*w));
    end
   
    end
    %predict common scores from gcca model
    predicted_t{k}=predict_gcca(local_t(idx),model.forprediction{k}.predparamGCCA);
    predicted_t{k}(:,setdiff(1:size(predicted_t{k},2), model.options.nCompsForEachContribution{k}))=[];

    %orthogonalize all X-blocks and local_t
    for j=1:nBlocks
        X{j}.d=X{j}.d-predicted_t{k}*model.forprediction{k}.BorthX{j};
        local_t{j}.d=local_t{j}.d-predicted_t{k}*model.forprediction{k}.BorthT{j};
    end
    
    else %unique components
       xx=X{idx}.d;
    for j=1:model.options.nCompsForEachContribution{k}
         predicted_t{k}(:,j)=xx*model.forprediction{k}.W(:,j);
            xx=xx-predicted_t{k}(:,j)*model.forprediction{k}.P(:,j)';
        
    end
    
    %orthogonalise all blocks
    for j=1:nBlocks
        X{j}.d = X{j}.d-predicted_t{k}*model.forprediction{k}.Borth{j};
    end 
        
    end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Predict
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if isequal(model.options.preprocY,'autoscale')
    stdY=std(model.Y.d);
else
    stdY=ones(1,size(model.beta,2));
end

predictedY = (cell2mat(predicted_t)*model.beta).*repmat(stdY,nObj,1)+repmat(mean(model.Y.d),nObj,1);

%predict partial after adding each model contribution (for CV-ANOVA)
partial_predictedY=[];
partial_predictedY(:,:,1)=repmat(mean(model.Y.d),nObj,1);
beta=model.beta;
beta_idx=0;
%predict after each contribution
for i=1:length(model.options.blockCombinations)
    if length(model.options.blockCombinations{i})>1
    beta_idx=beta_idx+length(model.options.nCompsForEachContribution{i});
    else
           beta_idx=beta_idx+model.options.nCompsForEachContribution{i};
    end
 
    partial_beta=beta(1:beta_idx,:);
    partial_predictedY(:,:,end+1)=(cell2mat(predicted_t(1:i))*partial_beta).*repmat(stdY,nObj,1)+repmat(mean(model.Y.d),nObj,1);
end


    

